// Program Ŀ
//  Description: Scatter and Gather routines. Public Domain.                
//    File Name: SCATGATH.PRG                                               
//       Author: Dan Comeau 70451.2312@compuserve.com                       
//      Revisor: Dan Comeau                                                 
//      Changes: v1.1 - Fixed problem with Gather() to base loop on size of 
//                      array instead of number of fields.                  
// Date created: 02-16-95              Date updated: 03-08-96              
// Time created: 02:55:49pm            Time updated: 9:35:29PM             
//

#include "error.ch"

// Function Ŀ
//         Name: Scatter()                                                  
//  Description: Scatter data field name and values to an array to be used  
//               by Gather().                                               
//       Author: Dan Comeau                                                 
// Date created: 02-16-95              Date updated: 96/03/02              
// Time created: 02:55:49pm            Time updated: 5:27:24PM             
//      Example: TOTHISFILE->( Gather( FRTHISFILE->( Scatter() ) ) )        
//Ĵ
//    Arguments: None                                                       
// Return Value: aFields                                                    
//     See Also:                                                            
//
function Scatter()
  local nFieldCount := fcount()
  local aFields[ nFieldCount, 2 ]
  local nField

  for nField := 1 to nFieldCount
    // Using a fieldblock means that the gather file fields don't have to
    // be in the same order as the scattered file.
    aFields[ nField, 1 ] := fieldblock( fieldname( nField ) )
    aFields[ nField, 2 ] := fieldget(  nField )
  next

return aFields

// Function Ŀ
//         Name: Gather()                                                   
//  Description: Gather fields based on names and values from array as      
//               made by Scatter(). It handles fields in any order. It also 
//               handles fields that don't exist.                           
//       Author: Dan Comeau                                                 
// Date created: 02-16-95              Date updated: 03-08-96              
// Time created: 03:05:22pm            Time updated: 09:30pm               
//      Example: TOTHISFILE->( Gather( FRTHISFILE->( Scatter() ) ) )        
//Ĵ
//    Arguments: aFields                                                    
// Return Value: Nil                                                        
//     See Also:                                                            
//
function Gather( aFields )
  local nField
  local nFieldCount  := len( aFields )
  local bSaveHandler := errorblock()             // previous error handler

  errorblock( {|e| GatherError( e, bSaveHandler )} )   // new error handler

  for nField := 1 to nFieldCount
    begin sequence
      // update field using fieldblock made by scatter()
      eval( aFields[ nField, 1 ], aFields[ nField, 2 ] )
    end
  next

  errorblock( bSaveHandler )                     // restore default error handler

return Nil

// Function Ŀ
//         Name: GatherError()                                              
//  Description: Manage errors in Gather() for fields that don't exist.     
//       Author: Dan Comeau                                                 
// Date created: 96/01/28              Date updated: 96/01/28              
// Time created: 10:57:49AM            Time updated: 10:57:49AM            
//Ĵ
//    Arguments: e                                                          
//             : bOriginalErrBlock                                          
// Return Value: lResult                                                    
//     See Also:                                                            
//
function GatherError( e, bOriginalErrBlock )
  local lResult

  if e:genCode == EG_NOVAR                       // ignore missing field error
    break
  else                                           // Since this error is not handled here,
    lResult := eval( bOriginalErrBlock, e )      // pass the error along to the original error handler.
  endif

return lResult
